unit ClipUtils;

interface

uses graphics, Windows;

  function _CopyToClipboard(Handle: THandle; SelText: string; FontCharset: TFontCharset): boolean;
  function _PasteFromClipboard(Handle: THandle; FontCharset: TFontCharset; DeleteCRLF: boolean = true): string;

implementation

uses SysUtils;

function _CopyToClipboard(Handle: THandle; SelText: string; FontCharset: TFontCharset): boolean;
var
  Data: HGLOBAL;
  _Text: PChar;
  fmt: UINT;
begin
  Result := false;
  if SelText <> '' then
  begin
    Data := GlobalAlloc(GMEM_MOVEABLE or GMEM_ZEROINIT, length(SelText) + 1);
    if Data <> 0 then
    begin
      _Text := GlobalLock(Data);
      if Assigned(_Text) then
      begin
        StrCopy(_Text, PChar(SelText));
        GlobalUnlock(Data);
        if FontCharset = OEM_CHARSET then fmt := CF_OEMTEXT else fmt := CF_TEXT;
        if OpenClipboard(Handle) then
        begin
          EmptyClipboard;
          SetClipboardData(fmt, Data);
          CloseClipboard;
          Result := true;
        end
          else GlobalFree(Data);
      end
        else GlobalFree(Data);
    end;
  end;
end;

function _PasteFromClipboard(Handle: THandle; FontCharset: TFontCharset; DeleteCRLF: boolean = true): string;
var
  fmtText, fmtOEMText: boolean;
  Data: HGLOBAL;
  fmt: UINT;
  _Text: PChar;
  Txt: string;
  i: integer;
begin
  Result := '';
  fmtText := IsClipboardFormatAvailable(CF_TEXT);
  fmtOEMText := IsClipboardFormatAvailable(CF_OEMTEXT);
  if (fmtText) or (fmtOEMText) then
    if OpenClipboard(Handle) then
    begin
      if fmtOEMText then fmt := CF_OEMTEXT else fmt := CF_TEXT;
      Data := GetClipboardData(fmt);
      if Data <> 0 then
      begin
        _Text := GlobalLock(Data);
        if Assigned(_Text) then
        begin
          Txt := StrPas(_Text);
          if (FontCharset = OEM_CHARSET) and (fmt = CF_TEXT) then CharToOEM(PChar(Txt), PChar(Txt))
            else
          if (FontCharset = DEFAULT_CHARSET) and (fmt = CF_OEMTEXT) then OEMToChar(PChar(Txt), PChar(Txt));
          if DeleteCRLF then
          begin
            i := 1;
            while i <= length(Txt) do
              if Txt[i] in [#10, #13] then System.Delete(Txt, i, 1)
                else inc(i);
          end;
          Result := Txt;
          GlobalUnlock(Data);
        end;
      end;
      CloseClipboard;
    end;
end;

end.