
{*******************************************************************************
      TMemoEx v2.1
    (c) alexey kolesnichenko, 2001.
*******************************************************************************}

unit demomain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, MemoEx, Menus, ActnList, Mask, ComCtrls;

type
  TTestForm = class(TForm)
    TestMemo: TMemoEx;
    TestPane: TPanel;
    Label1: TLabel;
    edMargin: TEdit;
    btnSetMargin: TButton;
    cbReadOnly: TCheckBox;
    cbWordWrap: TCheckBox;
    cbHighlight: TCheckBox;
    cbStripInvisible: TCheckBox;
    SB: TStatusBar;
    cbCursorBeyondEOL: TCheckBox;
    procedure edMarginKeyPress(Sender: TObject; var Key: Char);
    procedure btnSetMarginClick(Sender: TObject);
    procedure cbReadOnlyClick(Sender: TObject);
    procedure cbWordWrapClick(Sender: TObject);
    procedure edMarginChange(Sender: TObject);
    procedure cbHighlightClick(Sender: TObject);
    procedure cbStripInvisibleClick(Sender: TObject);
    procedure TestMemoGetLineAttr(Sender: TObject; const Line: String;
      index: Integer; const SelAttrs: TSelAttrs; var Attrs: TLineAttrs);
    procedure FormCreate(Sender: TObject);
    procedure TestMemoWordClick(Sender: TObject; const WordText: String;
      WordStyle: Word);
    function TestMemoBlockOperation(Sender: TObject; MacroID: Integer;
      const Text: String): String;
    function TestMemoCaseConversion(Sender: TObject; Conversion: Byte;
      const Text: String): String;
    procedure TestMemoBreakLine(Sender: TObject; const Original: String;
      var _New: String);
    procedure TestMemoConcatLine(Sender: TObject; const Original: String;
      var _New: String);
    procedure TestMemoChangeStatus(Sender: TObject);
    procedure TestMemoMouseOver(Sender: TObject; WordStyle: Word;
      var _Cursor: TCursor);
    function TestMemoInsertBlock(Sender: TObject;
      var Text: String): Boolean;
    function TestMemoInsertMacro(Sender: TObject;
      MacroID: Integer): String;
    procedure TestMemoSaveBlock(Sender: TObject; const Text: String);
    procedure TestMemoTextInsert(Sender: TObject; var Text: String);
    procedure TestMemoSelectionChange(Sender: TObject);
    procedure TestMemoChangeClipboardState(Sender: TObject;
      const CanPaste: Boolean);
    procedure cbCursorBeyondEOLClick(Sender: TObject);
  private
    { Private declarations }
    UseHighlight: boolean;
{$IFDEF VER130}
    procedure MouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
{$ENDIF}
  public
    { Public declarations }
  end;

var
  TestForm: TTestForm;

implementation

{$R *.DFM}

const
  _STYLE_NORMAL         = 0;  //   
  _STYLE_INVISIBLE      = 1;  //   
  _STYLE_BOLD           = 2;  //   
  _STYLE_ITALIC         = 3;  //  
  _STYLE_UNDERLINE      = 4;  //   
  _STYLE_SPECIAL        = 5;  //   
  _STYLE_HOTSPOT        = 6;  //  "" 

  _STYLE_FIRST          = _STYLE_NORMAL;
  _STYLE_LAST           = _STYLE_HOTSPOT;

var
  Styles: array [_STYLE_FIRST.._STYLE_LAST] of TLineAttr;

procedure SetDefaultTextStyles;

  procedure SetDefaultTextStyle(ATextStyle: integer; FC, BC: TColor; Style: TFontStyles; ExStyle: byte);
  begin
    Styles[ATextStyle].FC := FC;
    Styles[ATextStyle].BC := BC;
    Styles[ATextStyle].Style := Style;
    Styles[ATextStyle].ex_style := ExStyle;
  end;

begin
  SetDefaultTextStyle(_STYLE_NORMAL, clWindowText, clWindow, [], 0);
  SetDefaultTextStyle(_STYLE_INVISIBLE, clWindow, clWindow, [], 0);
  SetDefaultTextStyle(_STYLE_BOLD, clWindowText, clWindow, [fsBold], 0);
  SetDefaultTextStyle(_STYLE_ITALIC, clWindowText, clWindow, [fsItalic], 0);
  SetDefaultTextStyle(_STYLE_UNDERLINE, clWindowText, clWindow, [fsUnderline], 0);
  SetDefaultTextStyle(_STYLE_SPECIAL, clMaroon, clWindow, [fsBold], 0);
  SetDefaultTextStyle(_STYLE_HOTSPOT, clBlue, clWindow, [fsUnderline], _STYLE_HOTSPOT);
end;

procedure TTestForm.FormCreate(Sender: TObject);
begin
  //   
  UseHighlight := true;
  //    
  SetDefaultTextStyles;
{$IFDEF VER130}
  OnMouseWheel := MouseWheel;
{$ENDIF}
end;

procedure TTestForm.edMarginKeyPress(Sender: TObject; var Key: Char);
begin
  //     
  if not (Key in [#8, '0'..'9']) then Key := #0;
end;

procedure TTestForm.edMarginChange(Sender: TObject);
begin
  //      !
  if edMargin.Text = '' then edMargin.Text := '75';
end;

procedure TTestForm.btnSetMarginClick(Sender: TObject);
begin
  try
    //     RightMargin
    TestMemo.RightMargin := StrToInt(edMargin.Text);
  except
    //    ...
    TestMemo.RightMargin := 75;
  end;
end;

procedure TTestForm.cbReadOnlyClick(Sender: TObject);
begin
  //  /      
  TestMemo.ReadOnly := cbReadOnly.Checked;
end;

procedure TTestForm.cbWordWrapClick(Sender: TObject);
begin
  //  /  
  TestMemo.WordWrap := cbWordWrap.Checked;
end;

procedure TTestForm.cbHighlightClick(Sender: TObject);
begin
  //  /  
  UseHighlight := cbHighlight.Checked;
  //    TestMemo    
  TestMemo.Invalidate2;
end;

procedure TTestForm.cbStripInvisibleClick(Sender: TObject);
begin
  //  /  
  TestMemo.StripInvisible := cbStripInvisible.Checked;
end;

procedure TTestForm.cbCursorBeyondEOLClick(Sender: TObject);
begin
  //  /     
  TestMemo.CursorBeyondEOL := cbCursorBeyondEOL.Checked;
end;

procedure TTestForm.TestMemoGetLineAttr(Sender: TObject;
  const Line: String; index: Integer; const SelAttrs: TSelAttrs;
  var Attrs: TLineAttrs);

  procedure SetTextStyle(const AIndex: integer; const AStyle: byte);
  begin
    //       ,   Color,
    //      Attr[AIndex].BC --     Color.
    Attrs[AIndex] := Styles[AStyle];
  end;

  function SpecialsFound: boolean;
  //      (   ,    '>'
  var
    i: integer;
  begin
    Result := false;
    for i := 1 to length(Line) do
      if i < 10 then
        if ord(Line[i]) = 62 then
        begin
          //      '>',   ,   ...
          Result := true;
          break;
        end
          else
      else break;
    //    
    if Result then
      for i := 1 to length(Line) do SetTextStyle(i - 1, _STYLE_SPECIAL);
  end;

var
  i, j, k, x1, x2: integer;
begin
  //      ,  ...
  if not UseHighlight then exit;
  //    ...
  if SpecialsFound then exit;
  j := _STYLE_NORMAL;                 //   ""
  x1 := 0;
  x2 := 0;
  for i := 1 to length(Line) do
  begin
    case ord(Line[i]) of
  32:                                 //    
      begin
        k := _STYLE_NORMAL;
        j := _STYLE_NORMAL;
        x1 := 0;
        x2 := 0;
      end;
  42:                                 //   "" (**)
      k := _STYLE_BOLD;
  47:                                 //   "" (//)
      k := _STYLE_ITALIC;
  95:                                 //   "" (__)
      k := _STYLE_UNDERLINE;
  126:                                //   " " (~~)
      k := _STYLE_HOTSPOT;
    else                              //   ""
      k := _STYLE_NORMAL;
    end;
    if k > _STYLE_NORMAL then
    begin
      if (j = _STYLE_NORMAL) or (j <> k) then
      begin
        x1 := i;
        j := k;
        x2 := 0;
      end
        else
      if j = k then x2 := i;
      if (x1 > 0) and (x2 - 1 >= x1 + 1) then
      begin
        //   
        for k := x1 + 1 to x2 - 1 do SetTextStyle(k - 1, j);
        if TestMemo.ReadOnly then
        begin
          SetTextStyle(x1 - 1, _STYLE_INVISIBLE);
          SetTextStyle(x2 - 1, _STYLE_INVISIBLE);
        end;
        j := _STYLE_NORMAL;
        x1 := 0;
        x2 := 0;
      end
        else
      if (x1 > 0) and (x2 > 0) then
      begin
        //  ...
        x1 := 0;
        x2 := 0;
      end;
    end;
  end;
end;

procedure TTestForm.TestMemoWordClick(Sender: TObject;
  const WordText: String; WordStyle: Word);
begin
  //  WordClick --     ex_style <> 0
  //  WordStyle -- ex_style  
  //  WordText --  ,   
  //      ,   
  //       = _STYLE_HOTSPOT (.. ),
  //     ...
  if WordStyle = _STYLE_HOTSPOT then MessageDlg(Format('"%s" clicked!', [WordText]), mtInformation, [mbOk], 0);
  //          ...
end;

function TTestForm.TestMemoBlockOperation(Sender: TObject;
  MacroID: Integer; const Text: String): String;
begin
  //     
  //     [^b, ]   
  MessageDlg(Format('Process with: "%s"', [Text]), mtInformation, [mbOk], 0);
  Result := Text;
end;

function TTestForm.TestMemoCaseConversion(Sender: TObject;
  Conversion: Byte; const Text: String): String;
begin
  //      
  //   :
  //    ^k^o --   
  //    ^k^n --   
  //    ^o^u --  
  //     ,    
  //    -
  case Conversion of
RA_CASE_CONVERT_UPPER:
      Result := LowerCase(Text);  //  funny ;)
RA_CASE_CONVERT_LOWER:
      Result := UpperCase(Text);  //  funny ;)
RA_CASE_CONVERT_INVERT:
      Result := 'InVeRtEd CaSe';  //  funny ;)
  end;
end;

procedure TTestForm.TestMemoBreakLine(Sender: TObject;
  const Original: String; var _New: String);
begin
  //   ""  
  if pos('>', TrimLeft(Original)) > 0 then _New := '> ' + _New;
end;

procedure TTestForm.TestMemoConcatLine(Sender: TObject;
  const Original: String; var _New: String);
begin
  //   ""  
  if pos('>', TrimLeft(Original)) = pos('>', TrimLeft(_New)) then
    System.Delete(_New, 1, pos('>', TrimLeft(_New)) + 1);
end;

procedure TTestForm.TestMemoMouseOver(Sender: TObject; WordStyle: Word;
  var _Cursor: TCursor);
begin
  //     - 
  if WordStyle = _STYLE_HOTSPOT then _Cursor := crHandPoint;
end;

function TTestForm.TestMemoInsertBlock(Sender: TObject;
  var Text: String): Boolean;
begin
  //      -  (  clipboard)
  //  hotkey: ^k^r
  MessageDlg('Provide your "insert block" code here!', mtInformation, [mbOk], 0);
  Text := 'some text inserted';
  Result := true;
end;

function TTestForm.TestMemoInsertMacro(Sender: TObject;
  MacroID: Integer): String;
begin
  //   ,   
  //  : ^s,   
  Result := Format('Macro%d', [MacroID]);
end;

procedure TTestForm.TestMemoSaveBlock(Sender: TObject; const Text: String);
begin
  //       
  //  hotkey: ^k^w   
  MessageDlg(Format('Text "%s" saved!', [Text]), mtInformation, [mbOk], 0);
end;

procedure TTestForm.TestMemoTextInsert(Sender: TObject; var Text: String);
begin
  //      
  MessageDlg(Format('Text "%s" will be inserted', [Text]), mtInformation, [mbOk], 0);
end;

procedure TTestForm.TestMemoChangeStatus(Sender: TObject);
begin
  //    memo
  with SB do
  begin
    Panels[0].Text := Format('%3d:%3d', [TestMemo.CaretY + 1, TestMemo.CaretX + 1]);
    if TestMemo.Recording then Panels[1].Text := 'recording'
      else if TestMemo.InsertMode then Panels[1].Text := 'insert'
        else Panels[1].Text := 'overwrite';
    if TestMemo.IsUndoEmpty then Panels[2].Text := 'can''t undo'
      else Panels[2].Text := 'can undo';
  end;
end;

procedure TTestForm.TestMemoSelectionChange(Sender: TObject);
begin
  //    /  
  if TestMemo.SelectedText then SB.Panels[3].Text := 'can do block operations'
    else SB.Panels[3].Text := 'can''t do block operations';
end;

procedure TTestForm.TestMemoChangeClipboardState(Sender: TObject;
  const CanPaste: Boolean);
begin
  //      clipboard
  if CanPaste then SB.Panels[4].Text := 'can paste text'
    else SB.Panels[4].Text := 'can''t paste text';
end;

{$IFDEF VER130}
procedure TTestForm.MouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; MousePos: TPoint; var Handled: Boolean);
begin
  if not TestMemo.Focused then
  begin
    //   TestMemo   ,      
    //   
    Handled := true;
    TestMemo.MouseWheelScroll(WheelDelta);
  end;
end;
{$ENDIF}

end.
